[PATCH v2] emacs: insert quotable parts in reply as they are displayed in show view

Subject: [PATCH v2] emacs: insert quotable parts in reply as they are displayed in show view

Date: Tue, 27 Aug 2013 22:50:44 +0300

To: notmuch@notmuchmail.org

Cc:

From: Jani Nikula


In reply, insert quotable parts using notmuch-show-insert-bodypart
instead of calling notmuch-mm-display-part-inline directly to render
the quoted parts as they are rendered in show view.

The notable change is that replies to text/calendar parts quote the
pretty printed output of icalendar-import-buffer rather than the ugly
raw vcalendar.
---
 emacs/notmuch-mua.el  |   13 ++++++++++---
 emacs/notmuch-show.el |   18 +++++++++++++-----
 2 files changed, 23 insertions(+), 8 deletions(-)

diff --git a/emacs/notmuch-mua.el b/emacs/notmuch-mua.el
index 2baae5f..c3aca80 100644
--- a/emacs/notmuch-mua.el
+++ b/emacs/notmuch-mua.el
@@ -28,6 +28,8 @@
 
 (eval-when-compile (require 'cl))
 
+(declare-function notmuch-show-insert-bodypart "notmuch-show" (msg part depth &optional hide))
+
 ;;
 
 (defcustom notmuch-mua-send-hook '(notmuch-mua-message-send-hook)
@@ -130,9 +132,11 @@ list."
 (defun notmuch-mua-insert-quotable-part (message part)
   (save-restriction
     (narrow-to-region (point) (point))
-    (notmuch-mm-display-part-inline message part (plist-get part :id)
-				    (plist-get part :content-type)
-				    notmuch-show-process-crypto)
+    ;; We don't want hooks, such as notmuch-wash-*, to be run on the
+    ;; quotable part.
+    (let ((notmuch-show-insert-text/plain-hook nil))
+      ;; Show the part but do not add buttons.
+      (notmuch-show-insert-bodypart message part 0 'no-buttons))
     (goto-char (point-max))))
 
 ;; There is a bug in emacs 23's message.el that results in a newline
@@ -219,6 +223,9 @@ list."
   (goto-char (point-max))
   (push-mark)
   (message-goto-body)
+
+  ;; Display message from the beginning if possible.
+  (recenter (- (line-number-at-pos) 1))
   (set-buffer-modified-p nil))
 
 (defun notmuch-mua-forward-message ()
diff --git a/emacs/notmuch-show.el b/emacs/notmuch-show.el
index 2896aae..904b98e 100644
--- a/emacs/notmuch-show.el
+++ b/emacs/notmuch-show.el
@@ -844,7 +844,11 @@ message at DEPTH in the current thread."
 (defun notmuch-show-insert-bodypart (msg part depth &optional hide)
   "Insert the body part PART at depth DEPTH in the current thread.
 
-If HIDE is non-nil then initially hide this part."
+HIDE determines whether to show or hide the part and the button
+as follows: If HIDE is nil, show the part and the button. If HIDE
+is t, hide the part initially and show the button. If HIDE is
+'no-buttons, show the part but do not add any buttons (this is
+useful for quoting in replies)."
 
   (let* ((content-type (downcase (plist-get part :content-type)))
 	 (mime-type (or (and (string= content-type "application/octet-stream")
@@ -854,15 +858,19 @@ If HIDE is non-nil then initially hide this part."
 			content-type))
 	 (nth (plist-get part :id))
 	 (beg (point))
-	 ;; We omit the part button for the first (or only) part if this is text/plain.
-	 (button (unless (and (string= mime-type "text/plain") (<= nth 1))
+	 ;; Hide the part initially if HIDE is t.
+	 (show-part (not (equal hide t)))
+	 ;; We omit the part button for the first (or only) part if
+	 ;; this is text/plain, or HIDE is 'no-buttons.
+	 (button (unless (or (equal hide 'no-buttons)
+			     (and (string= mime-type "text/plain") (<= nth 1)))
 		   (notmuch-show-insert-part-header nth mime-type content-type (plist-get part :filename))))
 	 (content-beg (point)))
 
     ;; Store the computed mime-type for later use (e.g. by attachment handlers).
     (plist-put part :computed-type mime-type)
 
-    (if (not hide)
+    (if show-part
         (notmuch-show-insert-bodypart-internal msg part mime-type nth depth button)
       (button-put button :notmuch-lazy-part
                   (list msg part mime-type nth depth button)))
@@ -875,7 +883,7 @@ If HIDE is non-nil then initially hide this part."
       (insert "\n"))
     ;; We do not create the overlay for hidden (lazy) parts until
     ;; they are inserted.
-    (if (not hide)
+    (if show-part
 	(notmuch-show-create-part-overlays button content-beg (point))
       (save-excursion
 	(notmuch-show-toggle-part-invisibility button)))
-- 
1.7.10.4


Thread: